/* binary.h: Win32 routines to load/save chunks of binary data
   Copyright (c) 2008 Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#define IDD_BINARY                  1300
#define IDC_BINARY_STATIC_FILENAME  1301
#define IDC_BINARY_STATIC_PATH      1302
#define IDC_BINARY_BUTTON_BROWSE    1303
#define IDC_BINARY_STATIC_START     1304
#define IDC_BINARY_EDIT_START       1305
#define IDC_BINARY_STATIC_LENGTH    1306
#define IDC_BINARY_EDIT_LENGTH      1307
