/* keysyms.c: UI keysym to Fuse input layer keysym mappings
   Copyright (c) 2000-2007 Philip Kendall, Matan Ziv-Av, Russell Marks
                           Fredrick Meunier, Catalin Mihaila, Stuart Brady

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from keysyms.dat by keysyms.pl.
   Do not edit unless you know what you're doing! */

#include "config.h"


#include "input.h"
#include "keyboard.h"

#include <ui/wii/wiikeysyms.h>

keysyms_map_t keysyms_map[] = {

  { WII_KEY_TAB,             INPUT_KEY_Tab          },
  { WII_KEY_RETURN,          INPUT_KEY_Return       },
  { WII_KEY_ESCAPE,          INPUT_KEY_Escape       },
  { WII_KEY_SPACE,           INPUT_KEY_space        },
  { WII_KEY_EXCLAM,          INPUT_KEY_exclam       },
  { WII_KEY_QUOTEDBL,        INPUT_KEY_quotedbl     },
  { WII_KEY_DOLLAR,          INPUT_KEY_dollar       },
  { WII_KEY_PERCENT,         INPUT_KEY_percent      },
  { WII_KEY_AMPERSAND,       INPUT_KEY_ampersand    },
  { WII_KEY_APOSTROPHE,      INPUT_KEY_apostrophe   },
  { WII_KEY_PARENLEFT,       INPUT_KEY_parenleft    },
  { WII_KEY_PARENRIGHT,      INPUT_KEY_parenright   },
  { WII_KEY_ASTERISK,        INPUT_KEY_asterisk     },
  { WII_KEY_PLUS,            INPUT_KEY_plus         },
  { WII_KEY_COMMA,           INPUT_KEY_comma        },
  { WII_KEY_MINUS,           INPUT_KEY_minus        },
  { WII_KEY_PERIOD,          INPUT_KEY_period       },
  { WII_KEY_SLASH,           INPUT_KEY_slash        },
  { WII_KEY_0,               INPUT_KEY_0            },
  { WII_KEY_1,               INPUT_KEY_1            },
  { WII_KEY_2,               INPUT_KEY_2            },
  { WII_KEY_3,               INPUT_KEY_3            },
  { WII_KEY_4,               INPUT_KEY_4            },
  { WII_KEY_5,               INPUT_KEY_5            },
  { WII_KEY_6,               INPUT_KEY_6            },
  { WII_KEY_7,               INPUT_KEY_7            },
  { WII_KEY_8,               INPUT_KEY_8            },
  { WII_KEY_9,               INPUT_KEY_9            },
  { WII_KEY_COLON,           INPUT_KEY_colon        },
  { WII_KEY_SEMICOLON,       INPUT_KEY_semicolon    },
  { WII_KEY_LESS,            INPUT_KEY_less         },
  { WII_KEY_EQUAL,           INPUT_KEY_equal        },
  { WII_KEY_GREATER,         INPUT_KEY_greater      },
  { WII_KEY_QUESTION,        INPUT_KEY_question     },
  { WII_KEY_AT,              INPUT_KEY_at           },
  { WII_KEY_A,               INPUT_KEY_A            },
  { WII_KEY_B,               INPUT_KEY_B            },
  { WII_KEY_C,               INPUT_KEY_C            },
  { WII_KEY_D,               INPUT_KEY_D            },
  { WII_KEY_E,               INPUT_KEY_E            },
  { WII_KEY_F,               INPUT_KEY_F            },
  { WII_KEY_G,               INPUT_KEY_G            },
  { WII_KEY_H,               INPUT_KEY_H            },
  { WII_KEY_I,               INPUT_KEY_I            },
  { WII_KEY_J,               INPUT_KEY_J            },
  { WII_KEY_K,               INPUT_KEY_K            },
  { WII_KEY_L,               INPUT_KEY_L            },
  { WII_KEY_M,               INPUT_KEY_M            },
  { WII_KEY_N,               INPUT_KEY_N            },
  { WII_KEY_O,               INPUT_KEY_O            },
  { WII_KEY_P,               INPUT_KEY_P            },
  { WII_KEY_Q,               INPUT_KEY_Q            },
  { WII_KEY_R,               INPUT_KEY_R            },
  { WII_KEY_S,               INPUT_KEY_S            },
  { WII_KEY_T,               INPUT_KEY_T            },
  { WII_KEY_U,               INPUT_KEY_U            },
  { WII_KEY_V,               INPUT_KEY_V            },
  { WII_KEY_W,               INPUT_KEY_W            },
  { WII_KEY_X,               INPUT_KEY_X            },
  { WII_KEY_Y,               INPUT_KEY_Y            },
  { WII_KEY_Z,               INPUT_KEY_Z            },
  { WII_KEY_BRACKETLEFT,     INPUT_KEY_bracketleft  },
  { WII_KEY_BACKSLASH,       INPUT_KEY_backslash    },
  { WII_KEY_BRACKETRIGHT,    INPUT_KEY_bracketright },
  { WII_KEY_ASCIICIRCUM,     INPUT_KEY_asciicircum  },
  { WII_KEY_DEAD_CIRCUMFLEX, INPUT_KEY_dead_circumflex },
  { WII_KEY_UNDERSCORE,      INPUT_KEY_underscore   },
  { WII_KEY_A,               INPUT_KEY_a            },
  { WII_KEY_B,               INPUT_KEY_b            },
  { WII_KEY_C,               INPUT_KEY_c            },
  { WII_KEY_D,               INPUT_KEY_d            },
  { WII_KEY_E,               INPUT_KEY_e            },
  { WII_KEY_F,               INPUT_KEY_f            },
  { WII_KEY_G,               INPUT_KEY_g            },
  { WII_KEY_H,               INPUT_KEY_h            },
  { WII_KEY_I,               INPUT_KEY_i            },
  { WII_KEY_J,               INPUT_KEY_j            },
  { WII_KEY_K,               INPUT_KEY_k            },
  { WII_KEY_L,               INPUT_KEY_l            },
  { WII_KEY_M,               INPUT_KEY_m            },
  { WII_KEY_N,               INPUT_KEY_n            },
  { WII_KEY_O,               INPUT_KEY_o            },
  { WII_KEY_P,               INPUT_KEY_p            },
  { WII_KEY_Q,               INPUT_KEY_q            },
  { WII_KEY_R,               INPUT_KEY_r            },
  { WII_KEY_S,               INPUT_KEY_s            },
  { WII_KEY_T,               INPUT_KEY_t            },
  { WII_KEY_U,               INPUT_KEY_u            },
  { WII_KEY_V,               INPUT_KEY_v            },
  { WII_KEY_W,               INPUT_KEY_w            },
  { WII_KEY_X,               INPUT_KEY_x            },
  { WII_KEY_Y,               INPUT_KEY_y            },
  { WII_KEY_Z,               INPUT_KEY_z            },
  { WII_KEY_BRACELEFT,       INPUT_KEY_braceleft    },
  { WII_KEY_BAR,             INPUT_KEY_bar          },
  { WII_KEY_BRACERIGHT,      INPUT_KEY_braceright   },
  { WII_KEY_ASCIITILDE,      INPUT_KEY_asciitilde   },
  { WII_KEY_BACKSPACE,       INPUT_KEY_BackSpace    },
  { WII_KEY_UP,              INPUT_KEY_Up           },
  { WII_KEY_DOWN,            INPUT_KEY_Down         },
  { WII_KEY_LEFT,            INPUT_KEY_Left         },
  { WII_KEY_RIGHT,           INPUT_KEY_Right        },
  { WII_KEY_INSERT,          INPUT_KEY_Insert       },
  { WII_KEY_DELETE,          INPUT_KEY_Delete       },
  { WII_KEY_HOME,            INPUT_KEY_Home         },
  { WII_KEY_END,             INPUT_KEY_End          },
  { WII_KEY_PAGE_UP,         INPUT_KEY_Page_Up      },
  { WII_KEY_PAGE_DOWN,       INPUT_KEY_Page_Down    },
  { WII_KEY_CAPS_LOCK,       INPUT_KEY_Caps_Lock    },
  { WII_KEY_F1,              INPUT_KEY_F1           },
  { WII_KEY_F2,              INPUT_KEY_F2           },
  { WII_KEY_F3,              INPUT_KEY_F3           },
  { WII_KEY_F4,              INPUT_KEY_F4           },
  { WII_KEY_F5,              INPUT_KEY_F5           },
  { WII_KEY_F6,              INPUT_KEY_F6           },
  { WII_KEY_F7,              INPUT_KEY_F7           },
  { WII_KEY_F8,              INPUT_KEY_F8           },
  { WII_KEY_F9,              INPUT_KEY_F9           },
  { WII_KEY_F10,             INPUT_KEY_F10          },
  { WII_KEY_F11,             INPUT_KEY_F11          },
  { WII_KEY_F12,             INPUT_KEY_F12          },

  { 0, 0 }			/* End marker: DO NOT MOVE! */

};

