/* memorybrowser.rc: the Win32 memory browser
   Copyright (c) 2008 Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#include "memorybrowser.h"

IDD_MEM DIALOGEX DISCARDABLE 0,0,371,264
CAPTION "Fuse - Memory Browser"
FONT 8,"MS Sans Serif"
STYLE WS_POPUP | WS_CAPTION | WS_VISIBLE | WS_SYSMENU
EXSTYLE WS_EX_APPWINDOW
BEGIN
  CONTROL	"", IDC_MEM_LV, "SysListView32", \
                WS_CHILD | WS_TABSTOP | WS_VISIBLE | LVS_REPORT | \
                LVS_SINGLESEL | LVS_SHOWSELALWAYS | LVS_NOSCROLL, \
                0,0,360,240, WS_EX_CLIENTEDGE
  SCROLLBAR     IDC_MEM_SB, 360,0,11,240, WS_CHILD | SBS_VERT
  PUSHBUTTON	"&Close", IDCLOSE, 315,245,51,14
END
