/* keysyms.c: UI keysym to Fuse input layer keysym mappings
   Copyright (c) 2000-2007 Philip Kendall, Matan Ziv-Av, Russell Marks
                           Fredrick Meunier, Catalin Mihaila, Stuart Brady

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from keysyms.dat by keysyms.pl.
   Do not edit unless you know what you're doing! */

#include "config.h"


#include "input.h"
#include "keyboard.h"


keysyms_map_t keysyms_map[] = {

  {  15, INPUT_KEY_Tab          },
  {  28, INPUT_KEY_Return       },
  {   1, INPUT_KEY_Escape       },
  {  57, INPUT_KEY_space        },
  {  43, INPUT_KEY_numbersign   },
  {  40, INPUT_KEY_apostrophe   },
  {  51, INPUT_KEY_comma        },
  {  12, INPUT_KEY_minus        },
  {  52, INPUT_KEY_period       },
  {  53, INPUT_KEY_slash        },
  {  11, INPUT_KEY_0            },
  {   2, INPUT_KEY_1            },
  {   3, INPUT_KEY_2            },
  {   4, INPUT_KEY_3            },
  {   5, INPUT_KEY_4            },
  {   6, INPUT_KEY_5            },
  {   7, INPUT_KEY_6            },
  {   8, INPUT_KEY_7            },
  {   9, INPUT_KEY_8            },
  {  10, INPUT_KEY_9            },
  {  39, INPUT_KEY_semicolon    },
  {  13, INPUT_KEY_equal        },
  {  30, INPUT_KEY_A            },
  {  48, INPUT_KEY_B            },
  {  46, INPUT_KEY_C            },
  {  32, INPUT_KEY_D            },
  {  18, INPUT_KEY_E            },
  {  33, INPUT_KEY_F            },
  {  34, INPUT_KEY_G            },
  {  35, INPUT_KEY_H            },
  {  23, INPUT_KEY_I            },
  {  36, INPUT_KEY_J            },
  {  37, INPUT_KEY_K            },
  {  38, INPUT_KEY_L            },
  {  50, INPUT_KEY_M            },
  {  49, INPUT_KEY_N            },
  {  24, INPUT_KEY_O            },
  {  25, INPUT_KEY_P            },
  {  16, INPUT_KEY_Q            },
  {  19, INPUT_KEY_R            },
  {  31, INPUT_KEY_S            },
  {  20, INPUT_KEY_T            },
  {  22, INPUT_KEY_U            },
  {  47, INPUT_KEY_V            },
  {  17, INPUT_KEY_W            },
  {  45, INPUT_KEY_X            },
  {  21, INPUT_KEY_Y            },
  {  44, INPUT_KEY_Z            },
  {  26, INPUT_KEY_bracketleft  },
  {  86, INPUT_KEY_backslash    },
  {  27, INPUT_KEY_bracketright },
  {  30, INPUT_KEY_a            },
  {  48, INPUT_KEY_b            },
  {  46, INPUT_KEY_c            },
  {  32, INPUT_KEY_d            },
  {  18, INPUT_KEY_e            },
  {  33, INPUT_KEY_f            },
  {  34, INPUT_KEY_g            },
  {  35, INPUT_KEY_h            },
  {  23, INPUT_KEY_i            },
  {  36, INPUT_KEY_j            },
  {  37, INPUT_KEY_k            },
  {  38, INPUT_KEY_l            },
  {  50, INPUT_KEY_m            },
  {  49, INPUT_KEY_n            },
  {  24, INPUT_KEY_o            },
  {  25, INPUT_KEY_p            },
  {  16, INPUT_KEY_q            },
  {  19, INPUT_KEY_r            },
  {  31, INPUT_KEY_s            },
  {  20, INPUT_KEY_t            },
  {  22, INPUT_KEY_u            },
  {  47, INPUT_KEY_v            },
  {  17, INPUT_KEY_w            },
  {  45, INPUT_KEY_x            },
  {  21, INPUT_KEY_y            },
  {  44, INPUT_KEY_z            },
  {  14, INPUT_KEY_BackSpace    },
  {  96, INPUT_KEY_KP_Enter     },
  { 103, INPUT_KEY_Up           },
  { 108, INPUT_KEY_Down         },
  { 105, INPUT_KEY_Left         },
  { 106, INPUT_KEY_Right        },
  { 110, INPUT_KEY_Insert       },
  { 111, INPUT_KEY_Delete       },
  { 102, INPUT_KEY_Home         },
  { 107, INPUT_KEY_End          },
  { 104, INPUT_KEY_Page_Up      },
  { 109, INPUT_KEY_Page_Down    },
  {  58, INPUT_KEY_Caps_Lock    },
  {  59, INPUT_KEY_F1           },
  {  60, INPUT_KEY_F2           },
  {  61, INPUT_KEY_F3           },
  {  62, INPUT_KEY_F4           },
  {  63, INPUT_KEY_F5           },
  {  64, INPUT_KEY_F6           },
  {  65, INPUT_KEY_F7           },
  {  66, INPUT_KEY_F8           },
  {  67, INPUT_KEY_F9           },
  {  68, INPUT_KEY_F10          },
  {  87, INPUT_KEY_F11          },
  {  88, INPUT_KEY_F12          },
  {  42, INPUT_KEY_Shift_L      },
  {  54, INPUT_KEY_Shift_R      },
  {  29, INPUT_KEY_Control_L    },
  {  97, INPUT_KEY_Control_R    },
  {  56, INPUT_KEY_Alt_L        },
  { 100, INPUT_KEY_Alt_R        },
  { 125, INPUT_KEY_Meta_L       },
  { 126, INPUT_KEY_Meta_R       },
  { 127, INPUT_KEY_Mode_switch  },

  { 0, 0 }			/* End marker: DO NOT MOVE! */

};

