/* sdl2_ui.h: Routines for dealing with the SDL 2 user interface
   Copyright (c) 2026 Fredrick Meunier

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 */

#ifndef FUSE_SDL2_UI_H
#define FUSE_SDL2_UI_H

void sdl2ui_set_cursor_visibility( int show_cursor );

#endif
