/* options.c: options dialog boxes
   Copyright (c) 2001-2017 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from options.dat by options.pl.
   Do not edit unless you know what you're doing! */

#include "config.h"

int
option_enumerate_media_phantom_typist_mode( void )
{
  return 0;
}

int
option_enumerate_sound_stereo_ay( void )
{
  return 0;
}

int
option_enumerate_sound_speaker_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_plus3a_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_plus3b_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_beta128a_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_beta128b_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_beta128c_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_beta128d_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_plusd1_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_plusd2_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_didaktik80a_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_didaktik80b_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_disciple1_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_disciple2_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_opus1_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_drive_opus2_type( void )
{
  return 0;
}

int
option_enumerate_diskoptions_disk_try_merge( void )
{
  return 0;
}

int
option_enumerate_movie_movie_compr( void )
{
  return 0;
}
