/* keysyms.c: UI keysym to Fuse input layer keysym mappings
   Copyright (c) 2000-2007 Philip Kendall, Matan Ziv-Av, Russell Marks
                           Fredrick Meunier, Catalin Mihaila, Stuart Brady

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from keysyms.dat by keysyms.pl.
   Do not edit unless you know what you're doing! */

#include "config.h"


#include "input.h"
#include "keyboard.h"

#include <gdk/gdkkeysyms.h>

keysyms_map_t keysyms_map[] = {

  { GDK_KEY_Tab,     INPUT_KEY_Tab          },
  { GDK_KEY_Return,  INPUT_KEY_Return       },
  { GDK_KEY_Escape,  INPUT_KEY_Escape       },
  { GDK_KEY_space,   INPUT_KEY_space        },
  { GDK_KEY_exclam,  INPUT_KEY_exclam       },
  { GDK_KEY_quotedbl, INPUT_KEY_quotedbl     },
  { GDK_KEY_numbersign, INPUT_KEY_numbersign   },
  { GDK_KEY_dollar,  INPUT_KEY_dollar       },
  { GDK_KEY_percent, INPUT_KEY_percent      },
  { GDK_KEY_ampersand, INPUT_KEY_ampersand    },
  { GDK_KEY_apostrophe, INPUT_KEY_apostrophe   },
  { GDK_KEY_parenleft, INPUT_KEY_parenleft    },
  { GDK_KEY_parenright, INPUT_KEY_parenright   },
  { GDK_KEY_asterisk, INPUT_KEY_asterisk     },
  { GDK_KEY_plus,    INPUT_KEY_plus         },
  { GDK_KEY_comma,   INPUT_KEY_comma        },
  { GDK_KEY_minus,   INPUT_KEY_minus        },
  { GDK_KEY_period,  INPUT_KEY_period       },
  { GDK_KEY_slash,   INPUT_KEY_slash        },
  { GDK_KEY_0,       INPUT_KEY_0            },
  { GDK_KEY_1,       INPUT_KEY_1            },
  { GDK_KEY_2,       INPUT_KEY_2            },
  { GDK_KEY_3,       INPUT_KEY_3            },
  { GDK_KEY_4,       INPUT_KEY_4            },
  { GDK_KEY_5,       INPUT_KEY_5            },
  { GDK_KEY_6,       INPUT_KEY_6            },
  { GDK_KEY_7,       INPUT_KEY_7            },
  { GDK_KEY_8,       INPUT_KEY_8            },
  { GDK_KEY_9,       INPUT_KEY_9            },
  { GDK_KEY_colon,   INPUT_KEY_colon        },
  { GDK_KEY_semicolon, INPUT_KEY_semicolon    },
  { GDK_KEY_less,    INPUT_KEY_less         },
  { GDK_KEY_equal,   INPUT_KEY_equal        },
  { GDK_KEY_greater, INPUT_KEY_greater      },
  { GDK_KEY_question, INPUT_KEY_question     },
  { GDK_KEY_at,      INPUT_KEY_at           },
  { GDK_KEY_A,       INPUT_KEY_A            },
  { GDK_KEY_B,       INPUT_KEY_B            },
  { GDK_KEY_C,       INPUT_KEY_C            },
  { GDK_KEY_D,       INPUT_KEY_D            },
  { GDK_KEY_E,       INPUT_KEY_E            },
  { GDK_KEY_F,       INPUT_KEY_F            },
  { GDK_KEY_G,       INPUT_KEY_G            },
  { GDK_KEY_H,       INPUT_KEY_H            },
  { GDK_KEY_I,       INPUT_KEY_I            },
  { GDK_KEY_J,       INPUT_KEY_J            },
  { GDK_KEY_K,       INPUT_KEY_K            },
  { GDK_KEY_L,       INPUT_KEY_L            },
  { GDK_KEY_M,       INPUT_KEY_M            },
  { GDK_KEY_N,       INPUT_KEY_N            },
  { GDK_KEY_O,       INPUT_KEY_O            },
  { GDK_KEY_P,       INPUT_KEY_P            },
  { GDK_KEY_Q,       INPUT_KEY_Q            },
  { GDK_KEY_R,       INPUT_KEY_R            },
  { GDK_KEY_S,       INPUT_KEY_S            },
  { GDK_KEY_T,       INPUT_KEY_T            },
  { GDK_KEY_U,       INPUT_KEY_U            },
  { GDK_KEY_V,       INPUT_KEY_V            },
  { GDK_KEY_W,       INPUT_KEY_W            },
  { GDK_KEY_X,       INPUT_KEY_X            },
  { GDK_KEY_Y,       INPUT_KEY_Y            },
  { GDK_KEY_Z,       INPUT_KEY_Z            },
  { GDK_KEY_bracketleft, INPUT_KEY_bracketleft  },
  { GDK_KEY_backslash, INPUT_KEY_backslash    },
  { GDK_KEY_bracketright, INPUT_KEY_bracketright },
  { GDK_KEY_asciicircum, INPUT_KEY_asciicircum  },
  { GDK_KEY_dead_circumflex, INPUT_KEY_dead_circumflex },
  { GDK_KEY_underscore, INPUT_KEY_underscore   },
  { GDK_KEY_a,       INPUT_KEY_a            },
  { GDK_KEY_b,       INPUT_KEY_b            },
  { GDK_KEY_c,       INPUT_KEY_c            },
  { GDK_KEY_d,       INPUT_KEY_d            },
  { GDK_KEY_e,       INPUT_KEY_e            },
  { GDK_KEY_f,       INPUT_KEY_f            },
  { GDK_KEY_g,       INPUT_KEY_g            },
  { GDK_KEY_h,       INPUT_KEY_h            },
  { GDK_KEY_i,       INPUT_KEY_i            },
  { GDK_KEY_j,       INPUT_KEY_j            },
  { GDK_KEY_k,       INPUT_KEY_k            },
  { GDK_KEY_l,       INPUT_KEY_l            },
  { GDK_KEY_m,       INPUT_KEY_m            },
  { GDK_KEY_n,       INPUT_KEY_n            },
  { GDK_KEY_o,       INPUT_KEY_o            },
  { GDK_KEY_p,       INPUT_KEY_p            },
  { GDK_KEY_q,       INPUT_KEY_q            },
  { GDK_KEY_r,       INPUT_KEY_r            },
  { GDK_KEY_s,       INPUT_KEY_s            },
  { GDK_KEY_t,       INPUT_KEY_t            },
  { GDK_KEY_u,       INPUT_KEY_u            },
  { GDK_KEY_v,       INPUT_KEY_v            },
  { GDK_KEY_w,       INPUT_KEY_w            },
  { GDK_KEY_x,       INPUT_KEY_x            },
  { GDK_KEY_y,       INPUT_KEY_y            },
  { GDK_KEY_z,       INPUT_KEY_z            },
  { GDK_KEY_braceleft, INPUT_KEY_braceleft    },
  { GDK_KEY_bar,     INPUT_KEY_bar          },
  { GDK_KEY_braceright, INPUT_KEY_braceright   },
  { GDK_KEY_asciitilde, INPUT_KEY_asciitilde   },
  { GDK_KEY_BackSpace, INPUT_KEY_BackSpace    },
  { GDK_KEY_KP_Enter, INPUT_KEY_KP_Enter     },
  { GDK_KEY_Up,      INPUT_KEY_Up           },
  { GDK_KEY_Down,    INPUT_KEY_Down         },
  { GDK_KEY_Left,    INPUT_KEY_Left         },
  { GDK_KEY_Right,   INPUT_KEY_Right        },
  { GDK_KEY_Insert,  INPUT_KEY_Insert       },
  { GDK_KEY_Delete,  INPUT_KEY_Delete       },
  { GDK_KEY_Home,    INPUT_KEY_Home         },
  { GDK_KEY_End,     INPUT_KEY_End          },
  { GDK_KEY_Page_Up, INPUT_KEY_Page_Up      },
  { GDK_KEY_Page_Down, INPUT_KEY_Page_Down    },
  { GDK_KEY_Caps_Lock, INPUT_KEY_Caps_Lock    },
  { GDK_KEY_F1,      INPUT_KEY_F1           },
  { GDK_KEY_F2,      INPUT_KEY_F2           },
  { GDK_KEY_F3,      INPUT_KEY_F3           },
  { GDK_KEY_F4,      INPUT_KEY_F4           },
  { GDK_KEY_F5,      INPUT_KEY_F5           },
  { GDK_KEY_F6,      INPUT_KEY_F6           },
  { GDK_KEY_F7,      INPUT_KEY_F7           },
  { GDK_KEY_F8,      INPUT_KEY_F8           },
  { GDK_KEY_F9,      INPUT_KEY_F9           },
  { GDK_KEY_F10,     INPUT_KEY_F10          },
  { GDK_KEY_F11,     INPUT_KEY_F11          },
  { GDK_KEY_F12,     INPUT_KEY_F12          },
  { GDK_KEY_Shift_L, INPUT_KEY_Shift_L      },
  { GDK_KEY_Shift_R, INPUT_KEY_Shift_R      },
  { GDK_KEY_Control_L, INPUT_KEY_Control_L    },
  { GDK_KEY_Control_R, INPUT_KEY_Control_R    },
  { GDK_KEY_Alt_L,   INPUT_KEY_Alt_L        },
  { GDK_KEY_Alt_R,   INPUT_KEY_Alt_R        },
  { GDK_KEY_Meta_L,  INPUT_KEY_Meta_L       },
  { GDK_KEY_Meta_R,  INPUT_KEY_Meta_R       },
  { GDK_KEY_Super_L, INPUT_KEY_Super_L      },
  { GDK_KEY_Super_R, INPUT_KEY_Super_R      },
  { GDK_KEY_Hyper_L, INPUT_KEY_Hyper_L      },
  { GDK_KEY_Hyper_R, INPUT_KEY_Hyper_R      },
  { GDK_KEY_Mode_switch, INPUT_KEY_Mode_switch  },

  { 0, 0 }			/* End marker: DO NOT MOVE! */

};

