/* binary.rc: Win32 routines to load/save chunks of binary data
   Copyright (c) 2008 Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#include "binary.h"

IDD_BINARY DIALOGEX 0,0,396,85
STYLE DS_3DLOOK | DS_CENTER | DS_MODALFRAME | DS_FIXEDSYS | WS_VISIBLE | WS_BORDER | WS_CAPTION | WS_DLGFRAME | WS_POPUP | WS_SYSMENU
EXSTYLE WS_EX_APPWINDOW
CAPTION "Fuse - Load Binary Data"
FONT 8,"Ms Shell Dlg 2",400,0,1
BEGIN
    CTEXT           "Filename",IDC_BINARY_STATIC_FILENAME,5,5,34,15,SS_CENTERIMAGE
    LTEXT           "",IDC_BINARY_STATIC_PATH,45,5,290,15,SS_CENTERIMAGE
    PUSHBUTTON      "Browse...",IDC_BINARY_BUTTON_BROWSE,340,5,50,14
    CTEXT           "Start",IDC_BINARY_STATIC_START,5,25,34,15,SS_CENTERIMAGE
    EDITTEXT        IDC_BINARY_EDIT_START,45,25,344,15,ES_AUTOHSCROLL
    CTEXT           "Length",IDC_BINARY_STATIC_LENGTH,5,45,34,15,SS_CENTERIMAGE
    EDITTEXT        IDC_BINARY_EDIT_LENGTH,45,45,344,15,ES_AUTOHSCROLL
    DEFPUSHBUTTON   "OK",IDOK,275,65,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,335,65,50,14
END
