/* select_template.rc: Machine Select dialog box
   Copyright (c) 2008 Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#include "select_template.h"

IDD_SELECT_DIALOG DIALOGEX 0,0,177,247
STYLE DS_MODALFRAME | DS_SETFONT | DS_CENTER | WS_BORDER | WS_CAPTION | WS_DLGFRAME | WS_POPUP | WS_SYSMENU
EXSTYLE WS_EX_APPWINDOW
CAPTION "Fuse - Select"
FONT 8,"Ms Shell Dlg 2",400,0,1
BEGIN
/* those will be generated at runtime on WM_INITDIALOG - easier to control the size and position */
/*    DEFPUSHBUTTON   "&OK",IDOK,6,193,75,23 */
/*    PUSHBUTTON      "&Cancel",IDCANCEL,90,193,75,23 */
END
