/* keysyms.c: UI keysym to Fuse input layer keysym mappings
   Copyright (c) 2000-2007 Philip Kendall, Matan Ziv-Av, Russell Marks
                           Fredrick Meunier, Catalin Mihaila, Stuart Brady

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from keysyms.dat by keysyms.pl.
   Do not edit unless you know what you're doing! */

#include "config.h"


#include "input.h"
#include "keyboard.h"

#include <X11/keysym.h>

keysyms_map_t keysyms_map[] = {

  { XK_Tab,         INPUT_KEY_Tab          },
  { XK_Return,      INPUT_KEY_Return       },
  { XK_Escape,      INPUT_KEY_Escape       },
  { XK_space,       INPUT_KEY_space        },
  { XK_exclam,      INPUT_KEY_exclam       },
  { XK_quotedbl,    INPUT_KEY_quotedbl     },
  { XK_numbersign,  INPUT_KEY_numbersign   },
  { XK_dollar,      INPUT_KEY_dollar       },
  { XK_percent,     INPUT_KEY_percent      },
  { XK_ampersand,   INPUT_KEY_ampersand    },
  { XK_apostrophe,  INPUT_KEY_apostrophe   },
  { XK_parenleft,   INPUT_KEY_parenleft    },
  { XK_parenright,  INPUT_KEY_parenright   },
  { XK_asterisk,    INPUT_KEY_asterisk     },
  { XK_plus,        INPUT_KEY_plus         },
  { XK_comma,       INPUT_KEY_comma        },
  { XK_minus,       INPUT_KEY_minus        },
  { XK_period,      INPUT_KEY_period       },
  { XK_slash,       INPUT_KEY_slash        },
  { XK_0,           INPUT_KEY_0            },
  { XK_1,           INPUT_KEY_1            },
  { XK_2,           INPUT_KEY_2            },
  { XK_3,           INPUT_KEY_3            },
  { XK_4,           INPUT_KEY_4            },
  { XK_5,           INPUT_KEY_5            },
  { XK_6,           INPUT_KEY_6            },
  { XK_7,           INPUT_KEY_7            },
  { XK_8,           INPUT_KEY_8            },
  { XK_9,           INPUT_KEY_9            },
  { XK_colon,       INPUT_KEY_colon        },
  { XK_semicolon,   INPUT_KEY_semicolon    },
  { XK_less,        INPUT_KEY_less         },
  { XK_equal,       INPUT_KEY_equal        },
  { XK_greater,     INPUT_KEY_greater      },
  { XK_question,    INPUT_KEY_question     },
  { XK_at,          INPUT_KEY_at           },
  { XK_A,           INPUT_KEY_A            },
  { XK_B,           INPUT_KEY_B            },
  { XK_C,           INPUT_KEY_C            },
  { XK_D,           INPUT_KEY_D            },
  { XK_E,           INPUT_KEY_E            },
  { XK_F,           INPUT_KEY_F            },
  { XK_G,           INPUT_KEY_G            },
  { XK_H,           INPUT_KEY_H            },
  { XK_I,           INPUT_KEY_I            },
  { XK_J,           INPUT_KEY_J            },
  { XK_K,           INPUT_KEY_K            },
  { XK_L,           INPUT_KEY_L            },
  { XK_M,           INPUT_KEY_M            },
  { XK_N,           INPUT_KEY_N            },
  { XK_O,           INPUT_KEY_O            },
  { XK_P,           INPUT_KEY_P            },
  { XK_Q,           INPUT_KEY_Q            },
  { XK_R,           INPUT_KEY_R            },
  { XK_S,           INPUT_KEY_S            },
  { XK_T,           INPUT_KEY_T            },
  { XK_U,           INPUT_KEY_U            },
  { XK_V,           INPUT_KEY_V            },
  { XK_W,           INPUT_KEY_W            },
  { XK_X,           INPUT_KEY_X            },
  { XK_Y,           INPUT_KEY_Y            },
  { XK_Z,           INPUT_KEY_Z            },
  { XK_bracketleft, INPUT_KEY_bracketleft  },
  { XK_backslash,   INPUT_KEY_backslash    },
  { XK_bracketright, INPUT_KEY_bracketright },
  { XK_asciicircum, INPUT_KEY_asciicircum  },
  { XK_dead_circumflex, INPUT_KEY_dead_circumflex },
  { XK_underscore,  INPUT_KEY_underscore   },
  { XK_a,           INPUT_KEY_a            },
  { XK_b,           INPUT_KEY_b            },
  { XK_c,           INPUT_KEY_c            },
  { XK_d,           INPUT_KEY_d            },
  { XK_e,           INPUT_KEY_e            },
  { XK_f,           INPUT_KEY_f            },
  { XK_g,           INPUT_KEY_g            },
  { XK_h,           INPUT_KEY_h            },
  { XK_i,           INPUT_KEY_i            },
  { XK_j,           INPUT_KEY_j            },
  { XK_k,           INPUT_KEY_k            },
  { XK_l,           INPUT_KEY_l            },
  { XK_m,           INPUT_KEY_m            },
  { XK_n,           INPUT_KEY_n            },
  { XK_o,           INPUT_KEY_o            },
  { XK_p,           INPUT_KEY_p            },
  { XK_q,           INPUT_KEY_q            },
  { XK_r,           INPUT_KEY_r            },
  { XK_s,           INPUT_KEY_s            },
  { XK_t,           INPUT_KEY_t            },
  { XK_u,           INPUT_KEY_u            },
  { XK_v,           INPUT_KEY_v            },
  { XK_w,           INPUT_KEY_w            },
  { XK_x,           INPUT_KEY_x            },
  { XK_y,           INPUT_KEY_y            },
  { XK_z,           INPUT_KEY_z            },
  { XK_braceleft,   INPUT_KEY_braceleft    },
  { XK_bar,         INPUT_KEY_bar          },
  { XK_braceright,  INPUT_KEY_braceright   },
  { XK_asciitilde,  INPUT_KEY_asciitilde   },
  { XK_BackSpace,   INPUT_KEY_BackSpace    },
  { XK_KP_Enter,    INPUT_KEY_KP_Enter     },
  { XK_Up,          INPUT_KEY_Up           },
  { XK_Down,        INPUT_KEY_Down         },
  { XK_Left,        INPUT_KEY_Left         },
  { XK_Right,       INPUT_KEY_Right        },
  { XK_Insert,      INPUT_KEY_Insert       },
  { XK_Delete,      INPUT_KEY_Delete       },
  { XK_Home,        INPUT_KEY_Home         },
  { XK_End,         INPUT_KEY_End          },
  { XK_Page_Up,     INPUT_KEY_Page_Up      },
  { XK_Page_Down,   INPUT_KEY_Page_Down    },
  { XK_Caps_Lock,   INPUT_KEY_Caps_Lock    },
  { XK_F1,          INPUT_KEY_F1           },
  { XK_F2,          INPUT_KEY_F2           },
  { XK_F3,          INPUT_KEY_F3           },
  { XK_F4,          INPUT_KEY_F4           },
  { XK_F5,          INPUT_KEY_F5           },
  { XK_F6,          INPUT_KEY_F6           },
  { XK_F7,          INPUT_KEY_F7           },
  { XK_F8,          INPUT_KEY_F8           },
  { XK_F9,          INPUT_KEY_F9           },
  { XK_F10,         INPUT_KEY_F10          },
  { XK_F11,         INPUT_KEY_F11          },
  { XK_F12,         INPUT_KEY_F12          },
  { XK_Shift_L,     INPUT_KEY_Shift_L      },
  { XK_Shift_R,     INPUT_KEY_Shift_R      },
  { XK_Control_L,   INPUT_KEY_Control_L    },
  { XK_Control_R,   INPUT_KEY_Control_R    },
  { XK_Alt_L,       INPUT_KEY_Alt_L        },
  { XK_Alt_R,       INPUT_KEY_Alt_R        },
  { XK_Meta_L,      INPUT_KEY_Meta_L       },
  { XK_Meta_R,      INPUT_KEY_Meta_R       },
  { XK_Super_L,     INPUT_KEY_Super_L      },
  { XK_Super_R,     INPUT_KEY_Super_R      },
  { XK_Hyper_L,     INPUT_KEY_Hyper_L      },
  { XK_Hyper_R,     INPUT_KEY_Hyper_R      },
  { XK_Mode_switch, INPUT_KEY_Mode_switch  },

  { 0, 0 }			/* End marker: DO NOT MOVE! */

};

