/* browse.c: tape browser dialog box
   Copyright (c) 2008 Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#include "browse.h"

IDD_BROWSE DIALOGEX 0,0,230,153
STYLE DS_3DLOOK | DS_CENTER | DS_MODALFRAME | DS_FIXEDSYS | WS_BORDER | WS_CAPTION | WS_DLGFRAME | WS_POPUP | WS_SYSMENU
EXSTYLE WS_EX_APPWINDOW
CAPTION "Fuse - Browse Tape"
FONT 8,"Ms Shell Dlg 2",400,0,1
BEGIN
  CONTROL         "",IDC_BROWSE_LV,"SysListView32", 
                  WS_CHILD | WS_VISIBLE | LVS_REPORT
                  | LVS_SINGLESEL | LVS_SHOWSELALWAYS,
                  0,0,225,120, WS_EX_CLIENTEDGE
  CTEXT           "Tape modified",IDC_BROWSE_MODIFIED,0,125,225,8,SS_CENTERIMAGE
  DEFPUSHBUTTON   "&Close",IDCLOSE,175,135,50,14
END

win32bmp_tape_marker       BITMAP "ui/win32/icons/tape_marker.bmp"
win32bmp_tape_marker_mask  BITMAP "ui/win32/icons/tape_marker_mask.bmp"
