/* about.rc: about dialog box
   Copyright (c) 2011 Philip Kendall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#include "about.h"

IDD_ABOUT DIALOGEX 0, 0, 186, 95
STYLE DS_3DLOOK | DS_CENTER | DS_MODALFRAME | DS_SHELLFONT | WS_CAPTION | WS_VISIBLE | WS_POPUP | WS_SYSMENU
EXSTYLE WS_EX_APPWINDOW
CAPTION "About fuse"
FONT 8,"Ms Shell Dlg 2",400,0,1
{
    CTEXT           "Fuse " VERSION, IDC_ABOUT_STATIC_VERSION, 12, 5, 155, 16, SS_CENTER
    CTEXT           "The Free Unix Spectrum Emulator", IDC_ABOUT_STATIC_NAME, 12, 24, 155, 8, SS_CENTER
    CTEXT           FUSE_COPYRIGHT, IDC_ABOUT_STATIC_COPYRIGHT, 12, 40, 155, 8, SS_CENTER
    CTEXT           PACKAGE_URL, IDC_ABOUT_STATIC_WEBSITE, 12, 56, 155, 8, SS_CENTER|SS_NOTIFY
    DEFPUSHBUTTON   "Close", IDCLOSE, 68, 77, 50, 14
}
