/* options.rc: options dialog boxes
   Copyright (c) 2001-2007 Philip Kendall, Stuart Brady

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from options.dat by options-resource.pl.
   Do not edit unless you know what you're doing! */

#include "options_internals.h"

IDD_OPT_GENERAL DIALOGEX 6,5,190,225
  CAPTION "Fuse - General Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  LTEXT "&Emulation speed",IDC_OPT_GENERAL_LABEL_EMULATION_SPEED,5,7,90,9
  EDITTEXT IDC_OPT_GENERAL_EMULATION_SPEED,100,5,85,13,ES_NUMBER
  LTEXT "F&rame rate (1:n)",IDC_OPT_GENERAL_LABEL_FRAME_RATE,5,21,90,9
  EDITTEXT IDC_OPT_GENERAL_FRAME_RATE,100,19,85,13,ES_NUMBER
  AUTOCHECKBOX "Issue &2 keyboard",IDC_OPT_GENERAL_ISSUE2,5,33,160,11
  AUTOCHECKBOX "Recrea&ted ZX Spectrum",IDC_OPT_GENERAL_RECREATED_SPECTRUM,5,45,160,11
  AUTOCHECKBOX "Use shift with &arrow keys",IDC_OPT_GENERAL_KEYBOARD_ARROWS_SHIFTED,5,57,160,11
  AUTOCHECKBOX "Allow &writes to ROM",IDC_OPT_GENERAL_WRITABLE_ROMS,5,69,160,11
  AUTOCHECKBOX "Late t&imings",IDC_OPT_GENERAL_LATE_TIMINGS,5,81,160,11
  AUTOCHECKBOX "&Z80 is CMOS",IDC_OPT_GENERAL_Z80_IS_CMOS,5,93,160,11
  AUTOCHECKBOX "RS-232 &handshake",IDC_OPT_GENERAL_RS232_HANDSHAKE,5,105,160,11
  AUTOCHECKBOX "Black and white T&V",IDC_OPT_GENERAL_BW_TV,5,117,160,11
  AUTOCHECKBOX "&PAL-TV use TV2x effect",IDC_OPT_GENERAL_PAL_TV2X,5,129,160,11
  AUTOCHECKBOX "Full &screen",IDC_OPT_GENERAL_FULL_SCREEN,5,141,160,11
  AUTOCHECKBOX "Show status&bar",IDC_OPT_GENERAL_STATUSBAR,5,153,160,11
  AUTOCHECKBOX "Snap &joystick prompt",IDC_OPT_GENERAL_JOY_PROMPT,5,165,160,11
  AUTOCHECKBOX "&Confirm actions",IDC_OPT_GENERAL_CONFIRM_ACTIONS,5,177,160,11
  AUTOCHECKBOX "A&uto-save settings",IDC_OPT_GENERAL_AUTOSAVE_SETTINGS,5,189,160,11
  DEFPUSHBUTTON "OK",IDOK,45,206,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,206,50,14
END


IDD_OPT_MEDIA DIALOGEX 6,5,190,141
  CAPTION "Fuse - Media Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  AUTOCHECKBOX "&Auto-load media",IDC_OPT_MEDIA_AUTO_LOAD,5,5,160,11
  LTEXT "&Phantom typist mode",IDC_OPT_MEDIA_LABEL_PHANTOM_TYPIST_MODE,5,19,90,9
  COMBOBOX IDC_OPT_MEDIA_PHANTOM_TYPIST_MODE,100,17,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  AUTOCHECKBOX "&Detect loaders",IDC_OPT_MEDIA_DETECT_LOADER,5,31,160,11
  AUTOCHECKBOX "&Fastloading",IDC_OPT_MEDIA_FASTLOAD,5,43,160,11
  AUTOCHECKBOX "Use &tape traps",IDC_OPT_MEDIA_TAPE_TRAPS,5,55,160,11
  AUTOCHECKBOX "Accelerate l&oaders",IDC_OPT_MEDIA_ACCELERATE_LOADER,5,67,160,11
  AUTOCHECKBOX "Use .s&lt traps",IDC_OPT_MEDIA_SLT_TRAPS,5,79,160,11
  LTEXT "&MDR cartridge len",IDC_OPT_MEDIA_LABEL_MDR_LEN,5,93,90,9
  EDITTEXT IDC_OPT_MEDIA_MDR_LEN,100,91,85,13,ES_NUMBER
  AUTOCHECKBOX "Random len&gth MDR cartridge",IDC_OPT_MEDIA_MDR_RANDOM_LEN,5,105,160,11
  DEFPUSHBUTTON "OK",IDOK,45,122,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,122,50,14
END


IDD_OPT_PERIPHERALS_GENERAL DIALOGEX 6,5,190,221
  CAPTION "Fuse - General Peripheral Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  AUTOCHECKBOX "&Kempston joystick",IDC_OPT_PERIPHERALS_GENERAL_JOY_KEMPSTON,5,5,160,11
  AUTOCHECKBOX "Kempston &mouse",IDC_OPT_PERIPHERALS_GENERAL_KEMPSTON_MOUSE,5,17,160,11
  AUTOCHECKBOX "&Swap mouse buttons",IDC_OPT_PERIPHERALS_GENERAL_MOUSE_SWAP_BUTTONS,5,29,160,11
  AUTOCHECKBOX "&Interface 1",IDC_OPT_PERIPHERALS_GENERAL_INTERFACE1,5,41,160,11
  AUTOCHECKBOX "In&terface 2",IDC_OPT_PERIPHERALS_GENERAL_INTERFACE2,5,53,160,11
  AUTOCHECKBOX "Mu&ltiface One",IDC_OPT_PERIPHERALS_GENERAL_MULTIFACE1,5,65,160,11
  AUTOCHECKBOX "Multiface 12&8",IDC_OPT_PERIPHERALS_GENERAL_MULTIFACE128,5,77,160,11
  AUTOCHECKBOX "Multiface &3",IDC_OPT_PERIPHERALS_GENERAL_MULTIFACE3,5,89,160,11
  AUTOCHECKBOX "Stealt&h Multiface One",IDC_OPT_PERIPHERALS_GENERAL_MULTIFACE1_STEALTH,5,101,160,11
  AUTOCHECKBOX "Emulate &printers",IDC_OPT_PERIPHERALS_GENERAL_PRINTER,5,113,160,11
  AUTOCHECKBOX "&ZX Printer",IDC_OPT_PERIPHERALS_GENERAL_ZXPRINTER,5,125,160,11
  AUTOCHECKBOX "Speccy&Boot interface",IDC_OPT_PERIPHERALS_GENERAL_SPECCYBOOT,5,137,160,11
  AUTOCHECKBOX "Spectra&net",IDC_OPT_PERIPHERALS_GENERAL_SPECTRANET,5,149,160,11
  AUTOCHECKBOX "Spe&ctranet disable",IDC_OPT_PERIPHERALS_GENERAL_SPECTRANET_DISABLE,5,161,160,11
  AUTOCHECKBOX "TT&X2000S",IDC_OPT_PERIPHERALS_GENERAL_TTX2000S,5,173,160,11
  AUTOCHECKBOX "uSo&urce",IDC_OPT_PERIPHERALS_GENERAL_USOURCE,5,185,160,11
  DEFPUSHBUTTON "OK",IDOK,45,202,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,202,50,14
END


IDD_OPT_PERIPHERALS_SOUND DIALOGEX 6,5,190,89
  CAPTION "Fuse - Sound Peripheral Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  AUTOCHECKBOX "&Fuller Box",IDC_OPT_PERIPHERALS_SOUND_FULLER,5,5,160,11
  AUTOCHECKBOX "M&elodik",IDC_OPT_PERIPHERALS_SOUND_MELODIK,5,17,160,11
  AUTOCHECKBOX "Spec&Drum interface",IDC_OPT_PERIPHERALS_SOUND_SPECDRUM,5,29,160,11
  AUTOCHECKBOX "C&ovox interface",IDC_OPT_PERIPHERALS_SOUND_COVOX,5,41,160,11
  AUTOCHECKBOX "&uSpeech",IDC_OPT_PERIPHERALS_SOUND_USPEECH,5,53,160,11
  DEFPUSHBUTTON "OK",IDOK,45,70,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,70,50,14
END


IDD_OPT_PERIPHERALS_DISK DIALOGEX 6,5,190,233
  CAPTION "Fuse - Disk Peripheral Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  AUTOCHECKBOX "Simple &8-bit IDE",IDC_OPT_PERIPHERALS_DISK_SIMPLEIDE_ACTIVE,5,5,160,11
  AUTOCHECKBOX "&ZXATASP interface",IDC_OPT_PERIPHERALS_DISK_ZXATASP_ACTIVE,5,17,160,11
  AUTOCHECKBOX "ZXATASP up&load",IDC_OPT_PERIPHERALS_DISK_ZXATASP_UPLOAD,5,29,160,11
  AUTOCHECKBOX "Z&XATASP write protect",IDC_OPT_PERIPHERALS_DISK_ZXATASP_WP,5,41,160,11
  AUTOCHECKBOX "ZX&CF interface",IDC_OPT_PERIPHERALS_DISK_ZXCF_ACTIVE,5,53,160,11
  AUTOCHECKBOX "ZXCF &upload",IDC_OPT_PERIPHERALS_DISK_ZXCF_UPLOAD,5,65,160,11
  AUTOCHECKBOX "Div&IDE interface",IDC_OPT_PERIPHERALS_DISK_DIVIDE_ENABLED,5,77,160,11
  AUTOCHECKBOX "DivIDE &write protect",IDC_OPT_PERIPHERALS_DISK_DIVIDE_WP,5,89,160,11
  AUTOCHECKBOX "Div&MMC interface",IDC_OPT_PERIPHERALS_DISK_DIVMMC_ENABLED,5,101,160,11
  AUTOCHECKBOX "DivMMC w&rite protect",IDC_OPT_PERIPHERALS_DISK_DIVMMC_WP,5,113,160,11
  AUTOCHECKBOX "+&D interface",IDC_OPT_PERIPHERALS_DISK_PLUSD,5,125,160,11
  AUTOCHECKBOX "Dida&ktik 80 interface",IDC_OPT_PERIPHERALS_DISK_DIDAKTIK80,5,137,160,11
  AUTOCHECKBOX "DISCi&PLE interf(a)ce",IDC_OPT_PERIPHERALS_DISK_DISCIPLE,5,149,160,11
  AUTOCHECKBOX "&Beta 128 interface",IDC_OPT_PERIPHERALS_DISK_BETA128,5,161,160,11
  AUTOCHECKBOX "Beta 128 &auto-boot in 48K machines",IDC_OPT_PERIPHERALS_DISK_BETA128_48BOOT,5,173,160,11
  AUTOCHECKBOX "&Opus Discovery interface",IDC_OPT_PERIPHERALS_DISK_OPUS,5,185,160,11
  AUTOCHECKBOX "ZXMMC i&nterface",IDC_OPT_PERIPHERALS_DISK_ZXMMC_ENABLED,5,197,160,11
  DEFPUSHBUTTON "OK",IDOK,45,214,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,214,50,14
END


IDD_OPT_RZX DIALOGEX 6,5,190,91
  CAPTION "Fuse - RZX Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  AUTOCHECKBOX "Create &autosaves",IDC_OPT_RZX_RZX_AUTOSAVES,5,5,160,11
  AUTOCHECKBOX "&Compress RZX data",IDC_OPT_RZX_RZX_COMPRESSION,5,17,160,11
  AUTOCHECKBOX "C&ompetition mode",IDC_OPT_RZX_COMPETITION_MODE,5,29,160,11
  LTEXT "Co&mpetition code",IDC_OPT_RZX_LABEL_COMPETITION_CODE,5,43,90,9
  EDITTEXT IDC_OPT_RZX_COMPETITION_CODE,100,41,85,13,ES_NUMBER
  AUTOCHECKBOX "Always &embed snapshot",IDC_OPT_RZX_EMBED_SNAPSHOT,5,55,160,11
  DEFPUSHBUTTON "OK",IDOK,45,72,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,72,50,14
END


IDD_OPT_SOUND DIALOGEX 6,5,190,163
  CAPTION "Fuse - Sound Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  AUTOCHECKBOX "&Sound enabled",IDC_OPT_SOUND_SOUND,5,5,160,11
  AUTOCHECKBOX "&Loading sound",IDC_OPT_SOUND_SOUND_LOAD,5,17,160,11
  LTEXT "&AY stereo separation",IDC_OPT_SOUND_LABEL_STEREO_AY,5,31,90,9
  COMBOBOX IDC_OPT_SOUND_STEREO_AY,100,29,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  AUTOCHECKBOX "&Force 8-bit",IDC_OPT_SOUND_SOUND_FORCE_8BIT,5,43,160,11
  LTEXT "Speaker &type",IDC_OPT_SOUND_LABEL_SPEAKER_TYPE,5,57,90,9
  COMBOBOX IDC_OPT_SOUND_SPEAKER_TYPE,100,55,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "A&Y volume",IDC_OPT_SOUND_LABEL_VOLUME_AY,5,71,90,9
  EDITTEXT IDC_OPT_SOUND_VOLUME_AY,100,69,85,13,ES_NUMBER
  LTEXT "B&eeper volume",IDC_OPT_SOUND_LABEL_VOLUME_BEEPER,5,85,90,9
  EDITTEXT IDC_OPT_SOUND_VOLUME_BEEPER,100,83,85,13,ES_NUMBER
  LTEXT "Spec&Drum volume",IDC_OPT_SOUND_LABEL_VOLUME_SPECDRUM,5,99,90,9
  EDITTEXT IDC_OPT_SOUND_VOLUME_SPECDRUM,100,97,85,13,ES_NUMBER
  LTEXT "&Covox volume",IDC_OPT_SOUND_LABEL_VOLUME_COVOX,5,113,90,9
  EDITTEXT IDC_OPT_SOUND_VOLUME_COVOX,100,111,85,13,ES_NUMBER
  LTEXT "&uSpeech volume",IDC_OPT_SOUND_LABEL_VOLUME_USPEECH,5,127,90,9
  EDITTEXT IDC_OPT_SOUND_VOLUME_USPEECH,100,125,85,13,ES_NUMBER
  DEFPUSHBUTTON "OK",IDOK,45,144,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,144,50,14
END


IDD_OPT_DISKOPTIONS DIALOGEX 6,5,190,263
  CAPTION "Fuse - Drives Setup"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  LTEXT "+&3 Drive A",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_PLUS3A_TYPE,5,7,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_PLUS3A_TYPE,100,5,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "+3 Driv&e B",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_PLUS3B_TYPE,5,21,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_PLUS3B_TYPE,100,19,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  AUTOCHECKBOX "+3 Detect &Speedlock",IDC_OPT_DISKOPTIONS_PLUS3_DETECT_SPEEDLOCK,5,33,160,11
  LTEXT "Beta 128 Drive &A",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_BETA128A_TYPE,5,47,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_BETA128A_TYPE,100,45,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "Beta 128 Drive &B",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_BETA128B_TYPE,5,61,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_BETA128B_TYPE,100,59,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "Beta 128 Drive &C",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_BETA128C_TYPE,5,75,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_BETA128C_TYPE,100,73,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "Beta 128 Drive &D",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_BETA128D_TYPE,5,89,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_BETA128D_TYPE,100,87,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "+D Drive &1",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_PLUSD1_TYPE,5,103,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_PLUSD1_TYPE,100,101,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "+D Drive &2",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_PLUSD2_TYPE,5,117,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_PLUSD2_TYPE,100,115,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "Didaktik &80 Drive A",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_DIDAKTIK80A_TYPE,5,131,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_DIDAKTIK80A_TYPE,100,129,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "Dida&ktik 80 Drive B",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_DIDAKTIK80B_TYPE,5,145,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_DIDAKTIK80B_TYPE,100,143,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "DI&SCiPLE Drive 1",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_DISCIPLE1_TYPE,5,159,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_DISCIPLE1_TYPE,100,157,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "DISCiP&LE Drive 2",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_DISCIPLE2_TYPE,5,173,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_DISCIPLE2_TYPE,100,171,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "&Opus Drive 1",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_OPUS1_TYPE,5,187,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_OPUS1_TYPE,100,185,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "O&pus Drive 2",IDC_OPT_DISKOPTIONS_LABEL_DRIVE_OPUS2_TYPE,5,201,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DRIVE_OPUS2_TYPE,100,199,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  LTEXT "&Try merge 'B' side of disks",IDC_OPT_DISKOPTIONS_LABEL_DISK_TRY_MERGE,5,215,90,9
  COMBOBOX IDC_OPT_DISKOPTIONS_DISK_TRY_MERGE,100,213,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  AUTOCHECKBOX "Con&firm merge disk sides",IDC_OPT_DISKOPTIONS_DISK_ASK_MERGE,5,227,160,11
  DEFPUSHBUTTON "OK",IDOK,45,244,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,244,50,14
END


IDD_OPT_MOVIE DIALOGEX 6,5,190,55
  CAPTION "Fuse - Movie Options"
  FONT 8,"Ms Shell Dlg 2",400,0,1
  STYLE WS_POPUP | WS_CAPTION | WS_BORDER | WS_SYSMENU
  EXSTYLE WS_EX_APPWINDOW
BEGIN
  LTEXT "Movie &compression",IDC_OPT_MOVIE_LABEL_MOVIE_COMPR,5,7,90,9
  COMBOBOX IDC_OPT_MOVIE_MOVIE_COMPR,100,5,85,90,CBS_DROPDOWNLIST | CBS_HASSTRINGS
  AUTOCHECKBOX "&Stop recording after RZX ends",IDC_OPT_MOVIE_MOVIE_STOP_AFTER_RZX,5,19,160,11
  DEFPUSHBUTTON "OK",IDOK,45,36,50,14
  PUSHBUTTON "Cancel",IDCANCEL,100,36,50,14
END

