/* pokemem.h: Win32 interface to the poke memory
   Copyright (c) 2011 Philip Kendall, Sergio Baldoví

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#define IDD_POKEMEM       1900
#define IDC_PM_BANK       1901
#define IDC_PM_BANK_EDIT  1902
#define IDC_PM_ADDRESS    1903
#define IDC_PM_ADDR_EDIT  1904
#define IDC_PM_VALUE      1905
#define IDC_PM_VALUE_EDIT 1906
#define IDC_PM_ADD        1907
#define IDC_PM_ACTIVE_POK 1908
#define IDC_PM_LIST       1909
#define IDC_PM_LIST_EDIT  1910
