/* pokefinder.h: Win32 interface to the poke finder
   Copyright (c) 2004 Marek Januszwski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

#define IDD_POKEFINDER    1500
#define IDC_PF_SEARCH_FOR 1501
#define IDC_PF_EDIT       1502
#define IDC_PF_LOCATIONS  1503
#define IDC_PF_LIST       1504
#define IDC_PF_INC        1505
#define IDC_PF_DEC        1506
#define IDC_PF_SEARCH     1507
#define IDC_PF_RESET      1508
