/* menu_data.rc: menu structure for Fuse
   Copyright (c) 2004-2007 Philip Kendall, Stuart Brady, Marek Januszewski

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   Author contact information:

   E-mail: philip-fuse@shadowmagic.org.uk

*/

/* This file is autogenerated from menu_data.dat by ./menu_data.pl.
   Any changes made here will not be preserved. */

#include "config.h"

#include "menu.h"

#include <windows.h>
#include "ui/win32/menu_data.h"

win32_menu MENU
{
  POPUP "&File"
  {
    MENUITEM "&Open...\tF3", IDM_MENU_FILE_OPEN
    MENUITEM "&Save Snapshot...\tF2", IDM_MENU_FILE_SAVESNAPSHOT
    POPUP "&Recording"
    {
      MENUITEM "&Record...", IDM_MENU_FILE_RECORDING_RECORD
      MENUITEM "Record from s&napshot...", IDM_MENU_FILE_RECORDING_RECORDFROMSNAPSHOT
      MENUITEM "&Continue recording...", IDM_MENU_FILE_RECORDING_CONTINUERECORDING
      MENUITEM SEPARATOR
      MENUITEM "&Insert snapshot\tInsert", IDM_MENU_FILE_RECORDING_INSERTSNAPSHOT
      MENUITEM "Roll&back\tDelete", IDM_MENU_FILE_RECORDING_ROLLBACK
      MENUITEM SEPARATOR
      MENUITEM "&Play...", IDM_MENU_FILE_RECORDING_PLAY
      MENUITEM "&Stop", IDM_MENU_FILE_RECORDING_STOP
      MENUITEM "&Finalise...", IDM_MENU_FILE_RECORDING_FINALISE
    }
    POPUP "A&Y Logging"
    {
      MENUITEM "&Record...", IDM_MENU_FILE_AYLOGGING_RECORD
      MENUITEM "&Stop", IDM_MENU_FILE_AYLOGGING_STOP
    }
    MENUITEM SEPARATOR
    POPUP "S&creenshot"
    {
      MENUITEM "O&pen SCR Screenshot...", IDM_MENU_FILE_SCREENSHOT_OPENSCRSCREENSHOT
      MENUITEM "S&ave Screen as SCR...", IDM_MENU_FILE_SCREENSHOT_SAVESCREENASSCR
      MENUITEM "Op&en MLT Screenshot...", IDM_MENU_FILE_SCREENSHOT_OPENMLTSCREENSHOT
      MENUITEM "Sa&ve Screen as MLT...", IDM_MENU_FILE_SCREENSHOT_SAVESCREENASMLT
      MENUITEM "Save S&creen as PNG...", IDM_MENU_FILE_SCREENSHOT_SAVESCREENASPNG
    }
    POPUP "Scalable &Vector Graphics"
    {
      MENUITEM "Start capture in &line mode...", IDM_MENU_FILE_SCALABLEVECTORGRAPHICS_STARTCAPTUREINLINEMODE
      MENUITEM "Start capture in &dot mode...", IDM_MENU_FILE_SCALABLEVECTORGRAPHICS_STARTCAPTUREINDOTMODE
      MENUITEM "&Stop capture", IDM_MENU_FILE_SCALABLEVECTORGRAPHICS_STOPCAPTURE
    }
    POPUP "&Movie"
    {
      MENUITEM "&Record...", IDM_MENU_FILE_MOVIE_RECORD
      MENUITEM "R&ecord from RZX...", IDM_MENU_FILE_MOVIE_RECORDFROMRZX
      MENUITEM "&Pause", IDM_MENU_FILE_MOVIE_PAUSE
      MENUITEM "&Continue", IDM_MENU_FILE_MOVIE_CONTINUE
      MENUITEM "&Stop", IDM_MENU_FILE_MOVIE_STOP
    }
    MENUITEM SEPARATOR
    MENUITEM "Loa&d binary data...", IDM_MENU_FILE_LOADBINARYDATA
    MENUITEM "Save &binary data...", IDM_MENU_FILE_SAVEBINARYDATA
    MENUITEM SEPARATOR
    MENUITEM "E&xit...\tF10", IDM_MENU_FILE_EXIT
  }
  POPUP "&Options"
  {
    MENUITEM "&General...\tF4", IDM_MENU_OPTIONS_GENERAL
    MENUITEM "&Media...", IDM_MENU_OPTIONS_MEDIA
    MENUITEM "&Sound...", IDM_MENU_OPTIONS_SOUND
    POPUP "&Peripherals"
    {
      MENUITEM "&General...", IDM_MENU_OPTIONS_PERIPHERALS_GENERAL
      MENUITEM "&Sound...", IDM_MENU_OPTIONS_PERIPHERALS_SOUND
      MENUITEM "&Disk...", IDM_MENU_OPTIONS_PERIPHERALS_DISK
    }
    MENUITEM "&RZX...", IDM_MENU_OPTIONS_RZX
    MENUITEM "&Movie...", IDM_MENU_OPTIONS_MOVIE
    POPUP "&Joysticks"
    {
      MENUITEM "Joystick &1...", IDM_MENU_OPTIONS_JOYSTICKS_JOYSTICK1
      MENUITEM "Joystick &2...", IDM_MENU_OPTIONS_JOYSTICKS_JOYSTICK2
      MENUITEM "&Keyboard...", IDM_MENU_OPTIONS_JOYSTICKS_KEYBOARD
    }
    POPUP "S&elect ROMs"
    {
      POPUP "&Machine ROMs"
      {
        MENUITEM "Spectrum 1&6K...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM16K
        MENUITEM "Spectrum &48K...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM48K
        MENUITEM "Spectrum &128K...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM128K
        MENUITEM "Spectrum +&2...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM2
        MENUITEM "Spectrum +2&A...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM2A
        MENUITEM "Spectrum +&3...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM3
        MENUITEM "Spectrum +3&e...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUM3E
        MENUITEM "Timex &TC2048...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_TIMEXTC2048
        MENUITEM "Timex TC206&8...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_TIMEXTC2068
        MENUITEM "Ti&mex TS2068...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_TIMEXTS2068
        MENUITEM "&Pentagon 128K...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_PENTAGON128K
        MENUITEM "Pe&ntagon 512K...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_PENTAGON512K
        MENUITEM "Penta&gon 1024K...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_PENTAGON1024K
        MENUITEM "S&corpion ZS 256...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SCORPIONZS256
        MENUITEM "Spectrum &SE...", IDM_MENU_OPTIONS_SELECTROMS_MACHINEROMS_SPECTRUMSE
      }
      POPUP "&Peripheral ROMs"
      {
        MENUITEM "&Interface 1...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_INTERFACE1
        MENUITEM "&Beta 128...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_BETA128
        MENUITEM "+&D...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_D
        MENUITEM "Didaktik &80...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_DIDAKTIK80
        MENUITEM "DISCiP&LE...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_DISCIPLE
        MENUITEM "Mu&ltiface One...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_MULTIFACEONE
        MENUITEM "Multiface &128...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_MULTIFACE128
        MENUITEM "Multiface &3...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_MULTIFACE3
        MENUITEM "&Opus Discovery...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_OPUSDISCOVERY
        MENUITEM "&SP0256...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_SP0256
        MENUITEM "Specc&yBoot...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_SPECCYBOOT
        MENUITEM "TT&X2000S...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_TTX2000S
        MENUITEM "&uSource...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_USOURCE
        MENUITEM "uS&peech...", IDM_MENU_OPTIONS_SELECTROMS_PERIPHERALROMS_USPEECH
      }
    }
    MENUITEM "&Filter...", IDM_MENU_OPTIONS_FILTER
    MENUITEM "&Disk options...", IDM_MENU_OPTIONS_DISKOPTIONS
    MENUITEM SEPARATOR
    MENUITEM "S&ave", IDM_MENU_OPTIONS_SAVE
  }
  POPUP "&Machine"
  {
    MENUITEM "&Reset...\tF5", IDM_MENU_MACHINE_RESET
    MENUITEM "&Hard reset...", IDM_MENU_MACHINE_HARDRESET
    MENUITEM "&Select...\tF9", IDM_MENU_MACHINE_SELECT
    MENUITEM "&Debugger...", IDM_MENU_MACHINE_DEBUGGER
    MENUITEM "P&oke Finder...", IDM_MENU_MACHINE_POKEFINDER
    MENUITEM "Po&ke Memory...", IDM_MENU_MACHINE_POKEMEMORY
    MENUITEM "&Memory Browser...", IDM_MENU_MACHINE_MEMORYBROWSER
    POPUP "Pro&filer"
    {
      MENUITEM "&Start", IDM_MENU_MACHINE_PROFILER_START
      MENUITEM "&Stop", IDM_MENU_MACHINE_PROFILER_STOP
    }
    MENUITEM "&NMI", IDM_MENU_MACHINE_NMI
    MENUITEM "Multiface Red &Button", IDM_MENU_MACHINE_MULTIFACEREDBUTTON
    MENUITEM "Didaktik SNA&P", IDM_MENU_MACHINE_DIDAKTIKSNAP
  }
  POPUP "M&edia"
  {
    POPUP "&Tape"
    {
      MENUITEM "&Open...\tF7", IDM_MENU_MEDIA_TAPE_OPEN
      MENUITEM "&Play\tF8", IDM_MENU_MEDIA_TAPE_PLAY
      MENUITEM "&Browse...", IDM_MENU_MEDIA_TAPE_BROWSE
      MENUITEM "&Rewind", IDM_MENU_MEDIA_TAPE_REWIND
      MENUITEM "&Clear", IDM_MENU_MEDIA_TAPE_CLEAR
      MENUITEM "&Write...\tF6", IDM_MENU_MEDIA_TAPE_WRITE
      MENUITEM "R&ecord Start", IDM_MENU_MEDIA_TAPE_RECORDSTART
      MENUITEM "Recor&d Stop", IDM_MENU_MEDIA_TAPE_RECORDSTOP
    }
    POPUP "&Interface 1"
    {
      POPUP "Microdrive &1"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE1_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &2"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE2_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &3"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE3_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &4"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE4_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &5"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE5_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &6"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE6_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &7"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE7_WRITEPROTECT_DISABLE
        }
      }
      POPUP "Microdrive &8"
      {
        MENUITEM "Insert &New", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_INSERTNEW
        MENUITEM "&Insert...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_EJECT
        MENUITEM "&Save", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_SAVE
        MENUITEM "Save &As...", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_SAVEAS
        POPUP "Write &protect"
        {
          MENUITEM "&Enable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_WRITEPROTECT_ENABLE
          MENUITEM "&Disable", IDM_MENU_MEDIA_INTERFACE1_MICRODRIVE8_WRITEPROTECT_DISABLE
        }
      }
      POPUP "&RS232"
      {
        MENUITEM "Plug &RxD", IDM_MENU_MEDIA_INTERFACE1_RS232_PLUGRXD
        MENUITEM "&Unplug RxD", IDM_MENU_MEDIA_INTERFACE1_RS232_UNPLUGRXD
        MENUITEM "Plug &TxD", IDM_MENU_MEDIA_INTERFACE1_RS232_PLUGTXD
        MENUITEM "Un&plug TxD", IDM_MENU_MEDIA_INTERFACE1_RS232_UNPLUGTXD
      }
    }
    POPUP "&Disk"
    {
      POPUP "+&3"
      {
        POPUP "Drive &A:"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_3_DRIVEA_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_3_DRIVEA_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_3_DRIVEA_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_3_DRIVEA_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_3_DRIVEA_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_3_DRIVEA_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_3_DRIVEA_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_3_DRIVEA_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_3_DRIVEA_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &B:"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_3_DRIVEB_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_3_DRIVEB_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_3_DRIVEB_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_3_DRIVEB_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_3_DRIVEB_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_3_DRIVEB_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_3_DRIVEB_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_3_DRIVEB_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_3_DRIVEB_WRITEPROTECT_DISABLE
          }
        }
      }
      POPUP "&Beta"
      {
        POPUP "Drive &A:"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_BETA_DRIVEA_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &B:"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_BETA_DRIVEB_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &C:"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_BETA_DRIVEC_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &D:"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_BETA_DRIVED_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_BETA_DRIVED_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_BETA_DRIVED_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_BETA_DRIVED_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_BETA_DRIVED_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_BETA_DRIVED_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_BETA_DRIVED_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_BETA_DRIVED_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_BETA_DRIVED_WRITEPROTECT_DISABLE
          }
        }
      }
      POPUP "+&D"
      {
        POPUP "Drive &1"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_D_DRIVE1_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_D_DRIVE1_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_D_DRIVE1_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_D_DRIVE1_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_D_DRIVE1_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_D_DRIVE1_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_D_DRIVE1_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_D_DRIVE1_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_D_DRIVE1_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &2"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_D_DRIVE2_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_D_DRIVE2_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_D_DRIVE2_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_D_DRIVE2_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_D_DRIVE2_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_D_DRIVE2_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_D_DRIVE2_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_D_DRIVE2_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_D_DRIVE2_WRITEPROTECT_DISABLE
          }
        }
      }
      POPUP "DISC&iPLE"
      {
        POPUP "Drive &1"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE1_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &2"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_DISCIPLE_DRIVE2_WRITEPROTECT_DISABLE
          }
        }
      }
      POPUP "&Opus"
      {
        POPUP "Drive &1"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_OPUS_DRIVE1_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &2"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_OPUS_DRIVE2_WRITEPROTECT_DISABLE
          }
        }
      }
      POPUP "&Didaktik 80"
      {
        POPUP "Drive &A"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEA_WRITEPROTECT_DISABLE
          }
        }
        POPUP "Drive &B"
        {
          MENUITEM "Insert &New", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_INSERTNEW
          MENUITEM "&Insert...", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_INSERT
          MENUITEM "&Eject", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_EJECT
          MENUITEM "&Save", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_SAVE
          MENUITEM "Save &As...", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_SAVEAS
          POPUP "&Flip disk"
          {
            MENUITEM "Turn &upside down", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_FLIPDISK_TURNUPSIDEDOWN
            MENUITEM "Turn &back", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_FLIPDISK_TURNBACK
          }
          POPUP "Write &protect"
          {
            MENUITEM "&Enable", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_WRITEPROTECT_ENABLE
            MENUITEM "&Disable", IDM_MENU_MEDIA_DISK_DIDAKTIK80_DRIVEB_WRITEPROTECT_DISABLE
          }
        }
      }
    }
    POPUP "&Cartridge"
    {
      POPUP "&Timex Dock"
      {
        MENUITEM "&Insert...", IDM_MENU_MEDIA_CARTRIDGE_TIMEXDOCK_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_CARTRIDGE_TIMEXDOCK_EJECT
      }
      POPUP "&Interface 2"
      {
        MENUITEM "&Insert...", IDM_MENU_MEDIA_CARTRIDGE_INTERFACE2_INSERT
        MENUITEM "&Eject", IDM_MENU_MEDIA_CARTRIDGE_INTERFACE2_EJECT
      }
    }
    POPUP "ID&E"
    {
      POPUP "Simple &8-bit"
      {
        POPUP "&Master"
        {
          MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_SIMPLE8BIT_MASTER_INSERT
          MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_SIMPLE8BIT_MASTER_COMMIT
          MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_SIMPLE8BIT_MASTER_EJECT
        }
        POPUP "&Slave"
        {
          MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_SIMPLE8BIT_SLAVE_INSERT
          MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_SIMPLE8BIT_SLAVE_COMMIT
          MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_SIMPLE8BIT_SLAVE_EJECT
        }
      }
      POPUP "ZX&ATASP"
      {
        POPUP "&Master"
        {
          MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_ZXATASP_MASTER_INSERT
          MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_ZXATASP_MASTER_COMMIT
          MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_ZXATASP_MASTER_EJECT
        }
        POPUP "&Slave"
        {
          MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_ZXATASP_SLAVE_INSERT
          MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_ZXATASP_SLAVE_COMMIT
          MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_ZXATASP_SLAVE_EJECT
        }
      }
      POPUP "ZX&CF CompactFlash"
      {
        MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_ZXCFCOMPACTFLASH_INSERT
        MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_ZXCFCOMPACTFLASH_COMMIT
        MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_ZXCFCOMPACTFLASH_EJECT
      }
      POPUP "&DivIDE"
      {
        POPUP "&Master"
        {
          MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_DIVIDE_MASTER_INSERT
          MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_DIVIDE_MASTER_COMMIT
          MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_DIVIDE_MASTER_EJECT
        }
        POPUP "&Slave"
        {
          MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_DIVIDE_SLAVE_INSERT
          MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_DIVIDE_SLAVE_COMMIT
          MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_DIVIDE_SLAVE_EJECT
        }
      }
      POPUP "&DivMMC"
      {
        MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_DIVMMC_INSERT
        MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_DIVMMC_COMMIT
        MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_DIVMMC_EJECT
      }
      POPUP "&ZXMMC"
      {
        MENUITEM "&Insert...", IDM_MENU_MEDIA_IDE_ZXMMC_INSERT
        MENUITEM "&Commit", IDM_MENU_MEDIA_IDE_ZXMMC_COMMIT
        MENUITEM "&Eject", IDM_MENU_MEDIA_IDE_ZXMMC_EJECT
      }
    }
  }
  POPUP "&Help"
  {
    MENUITEM "&Keyboard...", IDM_MENU_HELP_KEYBOARD
    MENUITEM "&About...", IDM_MENU_HELP_ABOUT
  }
}

win32_accel ACCELERATORS
{
  VK_F3, IDM_MENU_FILE_OPEN, VIRTKEY
  VK_F2, IDM_MENU_FILE_SAVESNAPSHOT, VIRTKEY
  VK_INSERT, IDM_MENU_FILE_RECORDING_INSERTSNAPSHOT, VIRTKEY
  VK_DELETE, IDM_MENU_FILE_RECORDING_ROLLBACK, VIRTKEY
  VK_F10, IDM_MENU_FILE_EXIT, VIRTKEY
  VK_F4, IDM_MENU_OPTIONS_GENERAL, VIRTKEY
  VK_F5, IDM_MENU_MACHINE_RESET, VIRTKEY
  VK_F9, IDM_MENU_MACHINE_SELECT, VIRTKEY
  VK_F7, IDM_MENU_MEDIA_TAPE_OPEN, VIRTKEY
  VK_F8, IDM_MENU_MEDIA_TAPE_PLAY, VIRTKEY
  VK_F6, IDM_MENU_MEDIA_TAPE_WRITE, VIRTKEY
}
